==> armstrong-react-2.3.7.tgz: package/source/components/form/formBinders.tsx <==
import * as React from "react";
import * as _ from "underscore";
import { IFormBinder, IDataBinder, IFormBinderInjector, getFormBinderFromInjector, updateFormBinderInjector } from "./formCore";
import {
  IOneWayValueConverter,
  IValueConverter,
  IInputValueConverter,
  CheckboxValueConverter,
  NumericValueConverter,
  MultipleNumericValueConverter,
  INumericOptions,
  DefaultValueConverter
} from "./formValueConverters";
import { FormBinderBase } from "./formBinderBase";
import { ICalendarInputProps } from "./inputs/calendarInput";
import { IDateInputProps } from "./inputs/dateInput";
import { ITimeInputProps } from "./inputs/timeInput";
import { IAutoCompleteInputProps, IAutoCompleteOption } from "./inputs/autoCompleteInput";
import { Formatting } from "../../utilities/formatting";

/** An input FormBinder that sets native 'value' and 'onChange: (e) => void' properties */
export class InputFormBinder<TDataPropValue, TComponentPropValue> extends FormBinderBase<React.DOMAttributes<{}>, TDataPropValue, TComponentPropValue> {
  setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>) {
    super.setElementProperty(props, dataBinder)
    const v = props[this.propertySet];
    if (Formatting.isNullOrUndefined(v)) {
      props[this.propertySet] = this.getDefaultInputValue()
    }
  }

  protected getDefaultInputValue(): any {
    return ""
  }

  handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, e.currentTarget[this.propertyGet], notifyChanged);
    };
  }
}

export class SelectMultipleFormBinder<TDataProp> extends InputFormBinder<TDataProp, string[]>{
  constructor(dataPath: string, valueConverter?: IValueConverter<TDataProp, string[]>) {
    super(dataPath, "value", valueConverter)
  }

  setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>) {
    super.setElementProperty(props, dataBinder)
    props["multiple"] = true
  }

  handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, this.getSelectValues(e.currentTarget), notifyChanged);
    };
  }

  private getSelectValues(select) {
    const result: string[] = [];
    const options = select && select.options;
    if (!options) {
      return result
    }
    for (let i = 0; i < options.length; i++) {
      let opt = options[i];
      if (opt.selected) {
        result.push(opt.value);
      }
    }
    return result;
  }
}

export class CheckboxFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue>{
  constructor(dataPath: string, valueConverter?: IValueConverter<TDataPropValue, TComponentPropValue>) {
    super(dataPath, "checked", valueConverter)
  }
  protected getDefaultInputValue() {
    return false
  }
}

/** A radio input FormBinder */
export class RadioFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
  setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>) {
    props["name"] = this.dataPath;
    props[this.propertySet] = this.convert(dataBinder.getValue(this.dataPath)) === props[this.propertyGet];
  }
}

export class DateInputFormBinder extends FormBinderBase<IDateInputProps, string, string>{
  constructor(dataPath: string) {
    super(dataPath, "date")
  }

  handleValueChanged(props: IDateInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, e, notifyChanged);
    };
  }
}

export class TimeInputFormBinder extends FormBinderBase<ITimeInputProps, string, string>{
  constructor(dataPath: string) {
    super(dataPath, "time")
  }
  static customValue(dataName: string) {
    return new TimeInputFormBinder(dataName);
  }

  handleValueChanged(props: ITimeInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, e, notifyChanged);
    };
  }
}

export class CalendarInputFormBinder extends FormBinderBase<ICalendarInputProps, string, string>{
  constructor(dataPath: string) {
    super(dataPath, "date")
  }

  static customValue(dataName: string) {
    return new CalendarInputFormBinder(dataName);
  }

  handleValueChanged(props: ICalendarInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onDateChanged = (e) => this.onChanged(dataBinder, e, notifyChanged);
  }
}

export class AutoCompleteFormBinder implements IFormBinder<IAutoCompleteInputProps, any> {
  constructor(public dataPath: string, private getItemFromId?: (id: string) => IAutoCompleteOption) { }
  setElementProperty(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>): void {
    const value = dataBinder.getValue(this.dataPath);
    if (_.isArray(value)) {
      if (this.getItemFromId) {
        props.value = value.map(v => this.getItemFromId(v.id))
        return
      }
      props.value = props.options ? props.options.filter(o => value.indexOf(o.id) > -1) : []
      return
    }
    if (this.getItemFromId) {
      props.value = this.getItemFromId(value.id)
      return
    }

    props.value = props.options && props.options.filter(o => value === o.id)[0]
  }

  handleValueChanged(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void {
    props.onSelected = c => {
      if (_.isArray(c)) {
        dataBinder.setValue(this.dataPath, c.map(cc => cc.id))
      }
      else {
        dataBinder.setValue(this.dataPath, c.id)
      }
      notifyChanged()
    }
  }
}

/** Form Binder helpers */
export class FormBinder {
  /** bind a custom form binder */
  static custom<P>(formBinder: IFormBinder<P, any>): IFormBinderInjector<P> {
    return updateFormBinderInjector({} as any, formBinder)
  }

  /** bind to a 'hidden' input */
  static hidden<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "hidden", valueConverter)
  }

  /** bind a string property to a 'password' input */
  static password<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "password", valueConverter)
  }

  /** bind a string property to a 'text' input */
  static text<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "text", valueConverter)
  }

  /** bind a string property to a 'email' input */
  static textEmail<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "email", valueConverter)
  }

  static autoCompleteInput(dataName: string, getItemFromId?: (id: string) => IAutoCompleteOption) {
    return FormBinder.custom(new AutoCompleteFormBinder(dataName, getItemFromId));
  }

  /** bind a 'date' string property to a CalendarInput (e.g. YYYY-MM-DD) */
  static calendarInput(dataName: string) {
    return FormBinder.custom(new CalendarInputFormBinder(dataName));
  }

  /** bind a 'date' string property to a DateInput (e.g. YYYY-MM-DD) */
  static dateInput(dataName: string) {
    return FormBinder.custom(new DateInputFormBinder(dataName));
  }

  /** bind a 'time' string property to a TimeInput (e.g. HH:MM) */
  static timeInput(dataName: string) {
    return FormBinder.custom(new TimeInputFormBinder(dataName));
  }

  private static defaultInputFormBinder<TDataPropValue, TTo>(dataName: string, type: string, valueConverter?: IValueConverter<TDataPropValue, TTo>, propertySet = "value") {
    let adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, propertySet, valueConverter));
    adaptorInjector["type"] = type;
    return adaptorInjector;
  }

  /** bind a number property to a range */
  static range(dataName: string, options?: INumericOptions) {
    let adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, "value"));
    if (options) {
      adaptorInjector["min"] = options.min;
      adaptorInjector["max"] = options.max;
      adaptorInjector["step"] = options.step || 1;
    }
    return adaptorInjector;
  }

  /** uncontroller text input */
  static defaultText<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return FormBinder.custom(new InputFormBinder(dataName, "defaultValue", valueConverter, "value"));
  }

  /** bind a number property to a 'text' input */
  static textNumeric(dataName: string, options?: INumericOptions) {
    const converter = options ? new NumericValueConverter(options) : NumericValueConverter.instance;
    let adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, "defaultValue", converter, "value"));
    adaptorInjector["type"] = "number";
    adaptorInjector["onKeyDown"] = e => KeyboardHelper.numericKeyPress(e, options);
    if (options) {
      adaptorInjector["min"] = options.min
      adaptorInjector["max"] = options.max
    }
    return adaptorInjector;
  }

  /** bind a TDataPropValue property to a select */
  static selectCustom<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return FormBinder.custom(new InputFormBinder(dataName, "value", valueConverter));
  }

  /** bind a string property to a select */
  static select(dataName: string) {
    return FormBinder.selectCustom(dataName, DefaultValueConverter.instance);
  }

  /** bind a number property to a select */
  static selectNumeric(dataName: string) {
    return FormBinder.selectCustom(dataName, NumericValueConverter.instance);
  }

  /** bind a TDataPropValue[] property to a multi select */
  static selectMultipleCustom<TDataPropValue>(dataName: string, valueConverter?: IValueConverter<TDataPropValue, string[]>) {
    return FormBinder.custom(new SelectMultipleFormBinder(dataName, valueConverter));
  }

  /** bind a string[] property to a multi select */
  static selectMultiple(dataName: string) {
    return FormBinder.selectMultipleCustom(dataName);
  }

  /** bind a number[] property to a multi select */
  static selectMultipleNumeric(dataName: string) {
    return FormBinder.selectMultipleCustom(dataName, MultipleNumericValueConverter.instance);
  }

  /** bind a TDataPropValue property to a 'checkbox' input */
  static checkboxCustom<TDataPropValue>(/** helooo */dataName: string, valueConverter?: IValueConverter<TDataPropValue, boolean>) {
    let adaptorInjector = FormBinder.custom(new CheckboxFormBinder(dataName, valueConverter));
    adaptorInjector["type"] = "checkbox";
    return adaptorInjector;
    //return this.defaultInputFormBinder(dataName, "checkbox", valueConverter, "checked")
  }

  /** bind a boolean property to a 'checkbox' input */
  static checkbox(dataName: string) {
    return FormBinder.checkboxCustom(dataName);
  }

  /** bind a TDataPropValue property to a 'radio' input, using trueValue and falseValue equality testing */
  static checkboxConvert<TDataPropValue>(dataName: string, trueValue: TDataPropValue, falseValue: TDataPropValue) {
    return FormBinder.checkboxCustom(dataName, new CheckboxValueConverter(trueValue, falseValue));
  }

  /** bind a TDataPropValue property to a 'radio' input */
  static radioCustom<TDataPropValue>(dataName: string, value: string, valueConverter: IInputValueConverter<TDataPropValue>) {
    let adaptorInjector = FormBinder.custom(new RadioFormBinder(dataName, "checked", valueConverter, "value"));
    adaptorInjector["type"] = "radio";
    adaptorInjector["value"] = value;
    return adaptorInjector;
  }

  /** bind a string property to a 'radio' input */
  static radio(dataName: string, value: string) {
    return FormBinder.radioCustom(dataName, value, DefaultValueConverter.instance);
  }

  /** bind a number property to a 'radio' input */
  static radioNumeric(dataName: string, value: number) {
    return FormBinder.radioCustom(dataName, value.toString(), NumericValueConverter.instance);
  }
}

class KeyboardHelper {
  private static getNumericRegEx(options?: INumericOptions) {
    if (options) {
      if (options.allowNegative) {
        if (options.decimals) {
          return /[\d\.\-]/
        }
        return /[\d\-]/
      }

      if (options.decimals) {
        return /[\d\.]/
      }
    }

    return /\d/;
  }

  static numericKeyPress(e: React.KeyboardEvent<{}>, options?: INumericOptions) {
    const element = e.currentTarget as HTMLInputElement;
    let value = element.value;

    if (e.keyCode === 189 && value.indexOf("-") !== -1) {
      e.preventDefault();
      return;
    }
    if (e.keyCode === 190 && value.indexOf(".") !== -1) {
      e.preventDefault();
      return;
    }
  }
}

==> armstrong-react-2.3.12.tgz: package/source/components/form/formBinders.tsx <==
import * as React from "react";
import * as _ from "underscore";
import { IFormBinder, IDataBinder, IFormBinderInjector, getFormBinderFromInjector, updateFormBinderInjector } from "./formCore";
import {
  IOneWayValueConverter,
  IValueConverter,
  IInputValueConverter,
  CheckboxValueConverter,
  NumericValueConverter,
  MultipleNumericValueConverter,
  INumericOptions,
  DefaultValueConverter
} from "./formValueConverters";
import { FormBinderBase } from "./formBinderBase";
import { ICalendarInputProps } from "./inputs/calendarInput";
import { IDateInputProps } from "./inputs/dateInput";
import { ITimeInputProps } from "./inputs/timeInput";
import { IAutoCompleteInputProps, IAutoCompleteOption } from "./inputs/autoCompleteInput";
import { Formatting } from "../../utilities/formatting";

/** An input FormBinder that sets native 'value' and 'onChange: (e) => void' properties */
export class InputFormBinder<TDataPropValue, TComponentPropValue> extends FormBinderBase<React.DOMAttributes<{}>, TDataPropValue, TComponentPropValue> {
  setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>) {
    super.setElementProperty(props, dataBinder)
    const v = props[this.propertySet];
    if (Formatting.isNullOrUndefined(v)) {
      props[this.propertySet] = this.getDefaultInputValue()
    }
  }

  protected getDefaultInputValue(): any {
    return ""
  }

  handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, e.currentTarget[this.propertyGet], notifyChanged);
    };
  }
}

export class SelectMultipleFormBinder<TDataProp> extends InputFormBinder<TDataProp, string[]>{
  constructor(dataPath: string, valueConverter?: IValueConverter<TDataProp, string[]>) {
    super(dataPath, "value", valueConverter)
  }

  setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>) {
    super.setElementProperty(props, dataBinder)
    props["multiple"] = true
  }

  handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, this.getSelectValues(e.currentTarget), notifyChanged);
    };
  }

  private getSelectValues(select) {
    const result: string[] = [];
    const options = select && select.options;
    if (!options) {
      return result
    }
    for (let i = 0; i < options.length; i++) {
      let opt = options[i];
      if (opt.selected) {
        result.push(opt.value);
      }
    }
    return result;
  }
}

export class CheckboxFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue>{
  constructor(dataPath: string, valueConverter?: IValueConverter<TDataPropValue, TComponentPropValue>) {
    super(dataPath, "checked", valueConverter)
  }
  protected getDefaultInputValue() {
    return false
  }
}

/** A radio input FormBinder */
export class RadioFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
  setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>) {
    props["name"] = this.dataPath;
    props[this.propertySet] = this.convert(dataBinder.getValue(this.dataPath)) === props[this.propertyGet];
  }
}

export class DateInputFormBinder extends FormBinderBase<IDateInputProps, string, string>{
  constructor(dataPath: string) {
    super(dataPath, "date")
  }

  handleValueChanged(props: IDateInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, e, notifyChanged);
    };
  }
}

export class TimeInputFormBinder extends FormBinderBase<ITimeInputProps, string, string>{
  constructor(dataPath: string) {
    super(dataPath, "time")
  }
  static customValue(dataName: string) {
    return new TimeInputFormBinder(dataName);
  }

  handleValueChanged(props: ITimeInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onChange = (e) => {
      this.onChanged(dataBinder, e, notifyChanged);
    };
  }
}

export class CalendarInputFormBinder extends FormBinderBase<ICalendarInputProps, string, string>{
  constructor(dataPath: string) {
    super(dataPath, "date")
  }

  static customValue(dataName: string) {
    return new CalendarInputFormBinder(dataName);
  }

  handleValueChanged(props: ICalendarInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void) {
    props.onDateChanged = (e) => this.onChanged(dataBinder, e, notifyChanged);
  }
}

export class AutoCompleteFormBinder implements IFormBinder<IAutoCompleteInputProps, any> {
  constructor(public dataPath: string, private getItemFromId?: (id: string) => IAutoCompleteOption) { }
  setElementProperty(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>): void {
    const value = dataBinder.getValue(this.dataPath);
    if (_.isArray(value)) {
      if (this.getItemFromId) {
        props.value = value.map(v => this.getItemFromId(v))
        return
      }
      props.value = props.options ? props.options.filter(o => value.indexOf(o.id) > -1) : []
      return
    }
    if (this.getItemFromId) {
      props.value = this.getItemFromId(value)
      return
    }

    props.value = props.options && props.options.filter(o => value === o.id)[0]
  }

  handleValueChanged(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void {
    props.onSelected = c => {
      if (_.isArray(c)) {
        dataBinder.setValue(this.dataPath, c.map(cc => cc.id))
      }
      else {
        dataBinder.setValue(this.dataPath, c.id)
      }
      notifyChanged()
    }
  }
}

/** Form Binder helpers */
export class FormBinder {
  /** bind a custom form binder */
  static custom<P>(formBinder: IFormBinder<P, any>): IFormBinderInjector<P> {
    return updateFormBinderInjector({} as any, formBinder)
  }

  /** bind to a 'hidden' input */
  static hidden<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "hidden", valueConverter)
  }

  /** bind a string property to a 'password' input */
  static password<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "password", valueConverter)
  }

  /** bind a string property to a 'text' input */
  static text<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "text", valueConverter)
  }

  /** bind a string property to a 'email' input */
  static textEmail<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return this.defaultInputFormBinder(dataName, "email", valueConverter)
  }

  static autoCompleteInput(dataName: string, getItemFromId?: (id: string) => IAutoCompleteOption) {
    return FormBinder.custom(new AutoCompleteFormBinder(dataName, getItemFromId));
  }

  /** bind a 'date' string property to a CalendarInput (e.g. YYYY-MM-DD) */
  static calendarInput(dataName: string) {
    return FormBinder.custom(new CalendarInputFormBinder(dataName));
  }

  /** bind a 'date' string property to a DateInput (e.g. YYYY-MM-DD) */
  static dateInput(dataName: string) {
    return FormBinder.custom(new DateInputFormBinder(dataName));
  }

  /** bind a 'time' string property to a TimeInput (e.g. HH:MM) */
  static timeInput(dataName: string) {
    return FormBinder.custom(new TimeInputFormBinder(dataName));
  }

  private static defaultInputFormBinder<TDataPropValue, TTo>(dataName: string, type: string, valueConverter?: IValueConverter<TDataPropValue, TTo>, propertySet = "value") {
    let adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, propertySet, valueConverter));
    adaptorInjector["type"] = type;
    return adaptorInjector;
  }

  /** bind a number property to a range */
  static range(dataName: string, options?: INumericOptions) {
    let adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, "value"));
    if (options) {
      adaptorInjector["min"] = options.min;
      adaptorInjector["max"] = options.max;
      adaptorInjector["step"] = options.step || 1;
    }
    return adaptorInjector;
  }

  /** uncontroller text input */
  static defaultText<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return FormBinder.custom(new InputFormBinder(dataName, "defaultValue", valueConverter, "value"));
  }

  /** bind a number property to a 'text' input */
  static textNumeric(dataName: string, options?: INumericOptions) {
    const converter = options ? new NumericValueConverter(options) : NumericValueConverter.instance;
    let adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, "defaultValue", converter, "value"));
    adaptorInjector["type"] = "number";
    adaptorInjector["onKeyDown"] = e => KeyboardHelper.numericKeyPress(e, options);
    if (options) {
      adaptorInjector["min"] = options.min
      adaptorInjector["max"] = options.max
    }
    return adaptorInjector;
  }

  /** bind a TDataPropValue property to a select */
  static selectCustom<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>) {
    return FormBinder.custom(new InputFormBinder(dataName, "value", valueConverter));
  }

  /** bind a string property to a select */
  static select(dataName: string) {
    return FormBinder.selectCustom(dataName, DefaultValueConverter.instance);
  }

  /** bind a number property to a select */
  static selectNumeric(dataName: string) {
    return FormBinder.selectCustom(dataName, NumericValueConverter.instance);
  }

  /** bind a TDataPropValue[] property to a multi select */
  static selectMultipleCustom<TDataPropValue>(dataName: string, valueConverter?: IValueConverter<TDataPropValue, string[]>) {
    return FormBinder.custom(new SelectMultipleFormBinder(dataName, valueConverter));
  }

  /** bind a string[] property to a multi select */
  static selectMultiple(dataName: string) {
    return FormBinder.selectMultipleCustom(dataName);
  }

  /** bind a number[] property to a multi select */
  static selectMultipleNumeric(dataName: string) {
    return FormBinder.selectMultipleCustom(dataName, MultipleNumericValueConverter.instance);
  }

  /** bind a TDataPropValue property to a 'checkbox' input */
  static checkboxCustom<TDataPropValue>(/** helooo */dataName: string, valueConverter?: IValueConverter<TDataPropValue, boolean>) {
    let adaptorInjector = FormBinder.custom(new CheckboxFormBinder(dataName, valueConverter));
    adaptorInjector["type"] = "checkbox";
    return adaptorInjector;
    //return this.defaultInputFormBinder(dataName, "checkbox", valueConverter, "checked")
  }

  /** bind a boolean property to a 'checkbox' input */
  static checkbox(dataName: string) {
    return FormBinder.checkboxCustom(dataName);
  }

  /** bind a TDataPropValue property to a 'radio' input, using trueValue and falseValue equality testing */
  static checkboxConvert<TDataPropValue>(dataName: string, trueValue: TDataPropValue, falseValue: TDataPropValue) {
    return FormBinder.checkboxCustom(dataName, new CheckboxValueConverter(trueValue, falseValue));
  }

  /** bind a TDataPropValue property to a 'radio' input */
  static radioCustom<TDataPropValue>(dataName: string, value: string, valueConverter: IInputValueConverter<TDataPropValue>) {
    let adaptorInjector = FormBinder.custom(new RadioFormBinder(dataName, "checked", valueConverter, "value"));
    adaptorInjector["type"] = "radio";
    adaptorInjector["value"] = value;
    return adaptorInjector;
  }

  /** bind a string property to a 'radio' input */
  static radio(dataName: string, value: string) {
    return FormBinder.radioCustom(dataName, value, DefaultValueConverter.instance);
  }

  /** bind a number property to a 'radio' input */
  static radioNumeric(dataName: string, value: number) {
    return FormBinder.radioCustom(dataName, value.toString(), NumericValueConverter.instance);
  }
}

class KeyboardHelper {
  private static getNumericRegEx(options?: INumericOptions) {
    if (options) {
      if (options.allowNegative) {
        if (options.decimals) {
          return /[\d\.\-]/
        }
        return /[\d\-]/
      }

      if (options.decimals) {
        return /[\d\.]/
      }
    }

    return /\d/;
  }

  static numericKeyPress(e: React.KeyboardEvent<{}>, options?: INumericOptions) {
    const element = e.currentTarget as HTMLInputElement;
    let value = element.value;

    if (e.keyCode === 189 && value.indexOf("-") !== -1) {
      e.preventDefault();
      return;
    }
    if (e.keyCode === 190 && value.indexOf(".") !== -1) {
      e.preventDefault();
      return;
    }
  }
}
